@@ -29,6 +29,7 @@ def is_kivy_installed() -> bool:
 
 def gui() -> None:
     """ Entry points to launch GUI """
+
     # Check if kivy has been installed
     if not is_kivy_installed():
         import subprocess
@@ -46,6 +47,9 @@ def gui() -> None:
         subprocess.run(['gamut', '--gui', '--retry'])
         exit()
 
+    import os
+    os.environ["KIVY_NO_CONSOLELOG"] = "1"
+
     print_success('Launching user interface...')
     from .gui import GUI
     GUI().run()
@@ -58,7 +62,6 @@ def cli():
     from os import chdir, makedirs, remove, mkdir, getcwd
     from os.path import realpath, join, exists, splitext, basename
     from shutil import rmtree, copyfileobj
-    import pkg_resources
     from subprocess import run
     from pathlib import Path
     import json
@@ -291,7 +294,8 @@ def cli():
     # PRINT GAMUT PACKAGE VERSION
     # ------------------------------------- #
     if args.version:
-        print_success(f'GAMuT v{pkg_resources.get_distribution("gamut").version}')
+        from .__version__ import __version__
+        print_success(f'GAMuT v{__version__}')
 
     # ------------------------------------- #
     # OPEN GRAPHICAL USER INTERFACE
