@@ -8,7 +8,7 @@ from scipy import signal
 from typing_extensions import Self
 from collections.abc import Iterable
 
-from .config import AUDIO_FORMATS, CONSOLE
+from .config import AUDIO_FORMATS, CONSOLE, get_elapsed_time
 from .utils import resample_array
 from .controls import Envelope
 from .controls import Points
@@ -35,16 +35,20 @@ class AudioBuffer:
         self.bit_depth = bit_depth
 
     @catch_keyboard_interrupt(lambda: CONSOLE.log_process("\N{speaker with cancellation stroke} Audio stopped").print())
-    def play(self) -> None:
+    def play(self, blocking: bool = True) -> None:
         """ Plays back inner audio buffer """
         CONSOLE.log_process('\N{speaker}Playing audio...').print()
         try:
-            sd.play(self.y, samplerate=self.sr, blocking=True)
+            sd.play(self.y, samplerate=self.sr, blocking=blocking)
         except sd.PortAudioError:
             CONSOLE.error(
                 sd.PortAudioError,
                 f'Unable to play audio stream. It\'s possible the number of output channels in the audio source ({self.chans}) is greater than what your device supports.')
 
+    def stop(self) -> None:
+        """ Stops audio buffer when using ``blocking=True`` in play method """
+        sd.stop()
+
     def set_sampling_rate(self, sr: int) -> None:
         """ Sampling rate setter method """
         if self.sr != sr:
@@ -65,6 +69,7 @@ class AudioBuffer:
     def samps(self) -> int:
         return self.y.shape[0]
 
+    @get_elapsed_time
     def write(self, output_dir: str) -> None:
         """Writes audio to disk. This function is a simple wrapper of `sf.write()`"""
         ext = splitext(output_dir)[1]
