@@ -1,14 +1,14 @@
 from __future__ import annotations
 # librosa
 from librosa import magphase, stft, samples_like, load
-from librosa.feature import mfcc, chroma_stft, rms
+from librosa.feature import mfcc, chroma_stft, rms, zero_crossing_rate
 from librosa.beat import tempo
 
 # gamut
 from .controls import Points, Envelope, object_to_points
 from .utils import resample_array
 from .audio import AudioBuffer
-from .config import FILE_EXT, CONSOLE, ANALYSIS_TYPES, MIME_TYPES
+from .config import FILE_EXT, CONSOLE, ANALYSIS_TYPES, MIME_TYPES, AUDIO_DIR, get_elapsed_time
 from .data import KDTree
 
 # os
@@ -19,7 +19,8 @@ from os import walk, rename
 import filetype
 from random import choices, random
 from copy import deepcopy
-from time import time
+import datetime
+import os
 
 # typing
 from typing_extensions import Self
@@ -85,9 +86,9 @@ class Analyzer(ABC):
             print(f"{CONSOLE.c4}{key.upper()}: {CONSOLE.reset}{val}")
         print(f'{CONSOLE.c3}{line}{CONSOLE.reset}')
 
+    @get_elapsed_time
     def write(self, output: str, portable: bool = False) -> None:
         """ Writes a ``.gamut`` file to disk """
-        st = time()
         self.portable = portable
         self.name = splitext(basename(output))[0]
         output_dir = splitext(output)[0]
@@ -97,9 +98,9 @@ class Analyzer(ABC):
         # write file and set correct file extension
         np.save(output_dir, serialized_object)
         rename(output_dir + '.npy', output_dir+FILE_EXT)
-        CONSOLE.elapsed_time(st).print()
         return self
 
+    @get_elapsed_time
     def read(self, file: str, warn_user=False) -> Self:
         """ Reads a ``.gamut`` file from disk """
         if warn_user:
@@ -108,7 +109,7 @@ class Analyzer(ABC):
         # validate file
         if splitext(file)[1] != FILE_EXT:
             CONSOLE.error(ValueError, 'Wrong file extension. Provide a directory for a {} file'.format(FILE_EXT))
-        st = time()
+
         serialized_object = np.load(file, allow_pickle=True).item()
         if serialized_object['type'] != self.type:
             CONSOLE.error(TypeError, 'The specified file .gamut file is a {}, not a {}.'.format(
@@ -123,10 +124,9 @@ class Analyzer(ABC):
             if hasattr(self, attr):
                 setattr(self, attr, serialized_object[attr])
 
-        CONSOLE.elapsed_time(st).print()
         return self
 
-    def __get_type(self):
+    def __get_type(self) -> str:
         """ Helper function to get subclass name """
         return self.__class__.__name__.lower()
 
@@ -147,6 +147,9 @@ class Analyzer(ABC):
             analysis.extend(mfcc_features)
 
         if 'pitch' in features:
+            zerox = zero_crossing_rate(y=y,
+                                       frame_length=self.win_length,
+                                       hop_length=self.hop_length)
             loudness = rms(S=S,
                            frame_length=self.win_length,
                            hop_length=self.hop_length)
@@ -155,7 +158,7 @@ class Analyzer(ABC):
                                           n_fft=self.n_fft,
                                           win_length=self.win_length,
                                           hop_length=self.hop_length)
-            analysis.extend(np.concatenate([chroma_features, loudness]))
+            analysis.extend(np.concatenate([chroma_features, loudness, zerox]))
 
         analysis = np.array(analysis).T[:-1]
 
@@ -212,6 +215,7 @@ class Corpus(Analyzer):
         self.source = source
         self.max_duration = max_duration
         self.features = features
+        self.total_duration = 0
         self.source_root = ""
         self.tree = KDTree(leaf_size=leaf_size)
         CONSOLE.reset_counter('Analyzing audio samples: ')
@@ -228,31 +232,34 @@ class Corpus(Analyzer):
 
         self.soundfiles = []
         if self.source:
-            st = time()
+            self.source = [self.source] if isinstance(self.source, str) else self.source
+            self.source.append(os.path.join(AUDIO_DIR, 'silence.mp3'))
             self.__build()
-            CONSOLE.elapsed_time(st)
 
+    @get_elapsed_time
     def __build(self) -> None:
         """ build corpus from `source` """
-        st = time()
         CONSOLE.log_process('\N{brain} Building audio corpus...').print()
         data = self.__compile(source=None, data=[], excluded_files=[])
         CONSOLE.counter.finish()
         self.__set_source_root()
         self.tree.build(data=data, vector_path='features')
-        CONSOLE.elapsed_time(st).print()
 
     def _summarize(self) -> dict:
         filenames = "\n"
         for i, sf in enumerate(self.soundfiles):
+            if i == -1:
+                continue
             fn = basename(sf['file'])
             filenames += [f'\t{fn}', f', {fn}', f', {fn}\n'][[0, 1, 1, 2][i % 4]]
+
         return {
             "source root": self.source_root,
             "portable": self.portable,
+            "duration (H:M:S)": str(datetime.timedelta(seconds=int(self.total_duration))),
             "max. duration per source": f'{self.max_duration}' + ("s" if self.max_duration else ""),
             "max. tree leaf size": self.tree.leaf_size,
-            "analysis features": f'[{", ".join(self.features)}, ]',
+            "analysis features": ", ".join(self.features),
             f'sources ({len(self.soundfiles)})': filenames,
         }
 
@@ -287,6 +294,7 @@ class Corpus(Analyzer):
             'sr': sr,
             'y': y
         })
+        self.total_duration += len(y)/sr
         analysis, markers = self._analyze_audio_file(y=y, features=self.features, sr=sr)
         CONSOLE.counter.next()
         data.extend([{
@@ -432,12 +440,12 @@ class Mosaic(Analyzer):
             CONSOLE.error(ValueError, f'{corpus} is not a corpus')
         return corpora
 
+    @get_elapsed_time
     def __build(self, corpora: Iterable, sr: int | None = None) -> None:
         num_corpora = len(corpora)
         CONSOLE.log_process(
             f'\N{brain} Building mosaic for {basename(self.target)} from {"corpus" if num_corpora == 1 else f"{num_corpora} corpora"}...').print()
         CONSOLE.log_subprocess('Loading target...').print()
-        st = time()
         y, self.sr = load(self.target, sr=sr)
 
         self.duration = len(y) / self.sr
@@ -484,7 +492,6 @@ class Mosaic(Analyzer):
             self.frames.append([x['value'] for x in sorted(matches, key=lambda x: x['cost'])])
             CONSOLE.bar.next()
         CONSOLE.bar.finish()
-        CONSOLE.elapsed_time(st).print()
 
     def _serialize(self) -> dict:
         mosaic = deepcopy(vars(self))
@@ -497,12 +504,24 @@ class Mosaic(Analyzer):
         return mosaic
 
     def _summarize(self) -> dict:
+        duration = str(datetime.timedelta(seconds=int(self.duration))) if self.duration else None
+        num_corpora = 0
+        num_sources = []
+        for k in self.soundfiles:
+            if k < 0:
+                continue
+            num_corpora += 1
+            N = len(self.soundfiles[k]['sources'])
+            info = f"{N} source(s) from corpus {num_corpora}"
+            num_sources.append(info)
+
         return {
-            "target": basename(self.target) if self.target else None,
-            "num. of corpora": len([x for x in self.soundfiles]),
-            "analysis features": self.features,
+            "target": basename(str(self.target)),
             "portable": self.portable,
-            "duration": f'{round(self.duration * 10) / 10}s' if self.duration else None,
+            "duration (H:M:S)": duration,
+            "num. of corpora": num_corpora,
+            "num. of sources": ", ".join(num_sources),
+            "analysis features": ", ".join(self.features),
             "num. of grains": len(self.frames)
         }
 
@@ -529,6 +548,7 @@ class Mosaic(Analyzer):
                 CONSOLE.counter.next()
         CONSOLE.counter.finish()
 
+    @get_elapsed_time
     def to_audio(self,
                  # dynamic control parameters
                  fidelity: float | int | Envelope | Iterable = 1.0,
@@ -588,7 +608,7 @@ class Mosaic(Analyzer):
             """ resolve parameter into a ``Points`` instance based on type """
             return object_to_points(param, N)
 
-        def parse_corpus_weights_param(param):
+        def parse_corpus_weights_param(param: Envelope | float | int) -> np.ndarray:
             """ Generates corpus weights table based on param type """
             even_weights = True
             num_corpora = len(self.soundfiles.keys()) - 1
@@ -625,7 +645,6 @@ class Mosaic(Analyzer):
             CONSOLE.counter.finish()
             return soundfiles
 
-        st = time()
         CONSOLE.log_process(f'\N{brain} Generating audio from mosaic target: {basename(self.target)}...').print()
         # playback ratio
         sr, sr_ratio = (self.sr, 1) if not sr else (sr, sr/self.sr)
@@ -706,7 +725,6 @@ class Mosaic(Analyzer):
             CONSOLE.bar.next()
 
         CONSOLE.bar.finish()
-        CONSOLE.elapsed_time(st).print()
 
         # return normalized buffer
         return AudioBuffer(y=(buffer / np.amax(np.abs(buffer))) * np.sqrt(0.5), sr=sr)
